/*
  Copyright (C) 2001 Kimmo Pekkola

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
/*
  $Header: //rainbox/cvsroot/Rainmeter/Plugin/Rainmeter.cpp,v 1.6 2001/10/14 07:32:32 rainy Exp $

  $Log: Rainmeter.cpp,v $
  Revision 1.6  2001/10/14 07:32:32  rainy
  In error situations CError is thrown instead just a boolean value.

  Revision 1.5  2001/09/26 16:24:57  rainy
  Cleaned up the code

  Revision 1.4  2001/09/01 12:56:44  rainy
  Added refresh bang.

  Revision 1.3  2001/08/25 17:06:38  rainy
  Changed few methods to inlines.
  Now the wharf data is stored fully.

  Revision 1.2  2001/08/19 09:03:50  rainy
  Added support for Litestep's GetRevID.

  Revision 1.1.1.1  2001/08/11 10:58:19  Rainy
  Added to CVS.

*/

#include "Rainmeter.h"
#include "Error.h"
#include <assert.h>

CRainmeter* Rainmeter; // The module

bool CRainmeter::c_DummyLitestep=false;
LPCSTR CRainmeter::c_CmdLine=NULL;

/* 
** Interface to Litestep
*/

int initWharfModule(HWND ParentWnd, HINSTANCE dllInst, wharfDataType* wd)
{
	int Result=1;
	
	try {
		Rainmeter=new CRainmeter;

		if(Rainmeter) 
        {
			Result=Rainmeter->Initialize(ParentWnd, dllInst, wd, NULL);
		}

	} 
    catch(CError& error) 
    {
		MessageBox(ParentWnd, error.GetString().c_str(), APPNAME, MB_OK | MB_TOPMOST | MB_ICONEXCLAMATION);
	}

	return Result;
}


void quitWharfModule(HINSTANCE dllInst)
{
	quitModule(dllInst);
}

HRGN getLSRegion(int xOffset, int yOffset)
{
	if(Rainmeter) return Rainmeter->GetRegion(xOffset, yOffset);

	return NULL;
}

int initModuleEx(HWND ParentWnd, HINSTANCE dllInst, LPCSTR szPath)
{
	int Result=1;
	
	try {
		Rainmeter=new CRainmeter;

		if(Rainmeter) {
			Result=Rainmeter->Initialize(ParentWnd, dllInst, NULL, szPath);
		}

	} 
    catch(CError& error) 
    {
		MessageBox(ParentWnd, error.GetString().c_str(), APPNAME, MB_OK | MB_TOPMOST | MB_ICONEXCLAMATION);
	}

	return Result;
}

void quitModule(HINSTANCE dllInst)
{
	if(Rainmeter) 
	{
		Rainmeter->Quit(dllInst);
		delete Rainmeter;
		Rainmeter = NULL;
	}
}

void Initialize(bool DummyLS, LPCSTR CmdLine)
{
	CRainmeter::SetDummyLitestep(DummyLS);
	CRainmeter::SetCommandLine(CmdLine);

}

extern "C" DLLDECL void RainmeterHide(HWND, const char* arg)
{
	if(Rainmeter) Rainmeter->Hide();
}

extern "C" DLLDECL void RainmeterShow(HWND, const char* arg)
{
	if(Rainmeter) Rainmeter->Show();
}

extern "C" DLLDECL void RainmeterRefresh(HWND, const char* arg)
{
	if(Rainmeter) Rainmeter->Refresh();
}

/* 
** CRainmeter
**
** Constructor
**
*/
CRainmeter::CRainmeter()
{
	m_WharfData = NULL;
}

/* 
** ~CRainmeter
**
** Destructor
**
*/
CRainmeter::~CRainmeter()
{
	delete m_WharfData;
}

/* 
** Initialize
**
** The main initialization function for the module.
** May throw CErrors !!!!
**
*/
int CRainmeter::Initialize(HWND Parent, HINSTANCE Instance, wharfDataType* wd, LPCSTR szPath)
{
	int Result=0;

	if(Parent==NULL || Instance==NULL) 
	{
		throw CError(CError::ERROR_NULL_PARAMETER, __LINE__, __FILE__);
	}	

	if(wd!=NULL) {
		// Started as Wharf-module
		m_WharfData = new wharfDataType;
		memcpy(m_WharfData, wd, sizeof(wharfDataType));
	}

	// Create the meter
	Result = m_Meter.Initialize(*this, Parent, Instance);

	if(!c_DummyLitestep) 
	{
		int Msgs[] = { LM_GETREVID, 0 };
		// Register RevID message to Litestep
		::SendMessage(GetLitestepWnd(), LM_REGISTERMESSAGE, (WPARAM)m_Meter.GetWindow(), (LPARAM)Msgs);

		AddBangCommand("!RainmeterRefresh", RainmeterRefresh);
		AddBangCommand("!RainmeterHide", RainmeterHide);
		AddBangCommand("!RainmeterShow", RainmeterShow);
	}

	return Result;	// Alles OK
}

/* 
** Quit
**
** Called when the module quits
**
*/
void CRainmeter::Quit(HINSTANCE dllInst)
{
	if(!c_DummyLitestep) 
	{
		int Msgs[] = { LM_GETREVID, 0 };
		// Unregister RevID message
		if(m_Meter.GetWindow()) ::SendMessage(GetLitestepWnd(), LM_UNREGISTERMESSAGE, (WPARAM)m_Meter.GetWindow(), (LPARAM)Msgs);

		RemoveBangCommand("!RainmeterRefresh");
		RemoveBangCommand("!RainmeterHide");
		RemoveBangCommand("!RainmeterShow");
	}
}

/* 
** GetRegion
**
** Function returns the window region for the Wharf
**
*/
HRGN CRainmeter::GetRegion(int xOffset, int yOffset)
{
	return NULL;
}

